/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CUSTOMGAMEPLAYERS_H
#define CUSTOMGAMEPLAYERS_H

#include <GameInitSettings.h>

#include <GUI/StaticContainer.h>
#include <GUI/VBox.h>
#include <GUI/HBox.h>
#include <GUI/Label.h>
#include <GUI/DropDownBox.h>
#include <GUI/TextButton.h>
#include <GUI/ListBox.h>
#include <GUI/PictureLabel.h>
#include <GUI/Checkbox.h>
#include <GUI/TextView.h>
#include <GUI/TextBox.h>

#include <Network/ChangeEventList.h>

#include <DataTypes.h>

#include <string>
#include <list>

#include "MenuBase.h"

#define MENU_QUIT_GAME_FINISHED     (-2)

class INIFile;

class CustomGamePlayers : public MenuBase
{
public:
	CustomGamePlayers(const GameInitSettings& newGameInitSettings, bool server = true, bool LANServer = true);
	virtual ~CustomGamePlayers();

    void OnReceiveChangeEventList(ChangeEventList changeEventList);

    ChangeEventList GetChangeEventListForNewPlayer(std::string newPlayerName);

    virtual void update();

private:
    ChangeEventList GetChangeEventList();

    void OnReceiveChatMessage(std::string name, std::string message);
    void OnPeerDisconnected(std::string playername, bool bHost, int cause);

    void extractMapInfo(std::shared_ptr<INIFile>& pMap);

    void setPlayer2Slot(std::string playername, int slot);

    void OnChangeHousesDropDownBoxes(bool bInteractive, int houseInfoNum = -1);
    void OnChangeTeamDropDownBoxes(bool bInteractive, int houseInfoNum = -1);
    void OnChangePlayerDropDownBoxes(bool bInteractive, int boxnum);
    void OnClickPlayerDropDownBox(int boxnum);
    void OnStartGame(unsigned int timeLeft);
    void addToHouseDropDown(DropDownBox& houseDropDownBox, int house, bool bSelect = false);
    void removeFromHouseDropDown(DropDownBox& houseDropDownBox, int house);

    bool isBoundedHouseOnMap(HOUSETYPE houseID);

    void checkPlayerBoxes();
    void addAllPlayersToGameInitSettings();
    bool addPlayerToHouseInfo(GameInitSettings::HouseInfo& newHouseInfo, int player, std::string playername);

    void OnNext();
	void OnCancel();

	void OnSendChatMessage();
	void addInfoMessage(std::string message);
	void addChatMessage(std::string name, std::string message);

	void disableAllDropDownBoxes();

    GameInitSettings                gameInitSettings;
    GameInitSettings::HouseInfoList houseInfoListSetup;     ///< only used if we are loading a savegame

    StaticContainer WindowWidget;
    VBox			MainVBox;

    Label           CaptionLabel;

    HBox            MainHBox;

    // left VBox with player list and map options
    VBox            LeftVBox;
    HBox            PlayerListHBox;
    VBox            PlayerListVBox;

    // right VBox with mini map
    VBox            RightVBox;
    PictureLabel    Minimap;
    HBox            MapPropertiesHBox;
    VBox            MapPropertyNamesVBox;
    VBox            MapPropertyValuesVBox;
    Label           MapProperty_Size;
    Label           MapProperty_Players;
    Label           MapProperty_Authors;
    Label           MapProperty_License;

    // bottom row of buttons
    HBox            ButtonHBox;
    VBox            Button_BackVBox;
	TextButton		Button_Back;
	VBox            ChatVBox;
    TextView        ChatTextView;
    TextBox         ChatTextBox;
    VBox            Button_NextVBox;
    TextButton		Button_Next;

	class HouseInfo {
    public:
        VBox            HouseInfoVBox;
        HBox            HouseHBox;
        Label           HouseLabel;
        DropDownBox     HouseDropDown;
        DropDownBox     TeamDropDown;
        HBox            PlayerHBox;
        PictureLabel    Player1ArrowLabel;
        Label           Player1Label;
        DropDownBox     Player1DropDown;
        Label           Player2Label;
        DropDownBox     Player2DropDown;
	};

    bool                    bServer;
    bool                    bLANServer;
    HouseInfo               houseInfo[NUM_HOUSES];
    int                     numHouses;
    std::list<HOUSETYPE>    boundHousesOnMap;
    Uint32                  startGameTime;
};

#endif //CUSTOMGAMEPLAYERS_H
