#ifndef MULTIPLAYERMENU_H
#define MULTIPLAYERMENU_H

#include "MenuBase.h"

#include <GUI/Window.h>
#include <GUI/StaticContainer.h>
#include <GUI/HBox.h>
#include <GUI/TextButton.h>
#include <GUI/Spacer.h>
#include <GUI/Label.h>
#include <GUI/TextBox.h>
#include <GUI/SymbolButton.h>
#include <GUI/ScrollBar.h>
#include <GUI/ListBox.h>
#include <GUI/ProgressBar.h>
#include <GUI/PictureLabel.h>
#include <GUI/InvisibleButton.h>
#include <GUI/ClickMap.h>
#include <GUI/VBox.h>

#include <Network/LANGameFinderAndAnnouncer.h>
#include <Network/ChangeEventList.h>

#include <GameInitSettings.h>

#include <list>

class MultiPlayerMenu : public MenuBase {
public:
	MultiPlayerMenu();
	~MultiPlayerMenu();

	/**
		This method is called, when the child window is about to be closed.
		This child window will be closed after this method returns.
		\param	pChildWindow	The child window that will be closed
	*/
	virtual void OnChildWindowClose(Window* pChildWindow);

private:
    void showDisconnectMessageBox(int cause);

	void OnCreateLANGame();
	void OnCreateInternetGame();
	void OnConnect();
	void OnJoin();
	void OnQuit();

    void OnPeerDisconnected(std::string playername, bool bHost, int cause);

	void OnGameTypeChange(int buttonID);
	void OnGameListSelectionChange(bool bInteractive);

	void OnNewLANServer(GameServerInfo gameServerInfo);
	void OnUpdateLANServer(GameServerInfo gameServerInfo);
	void OnRemoveLANServer(GameServerInfo gameServerInfo);

	void OnGameServerInfoList(std::list<GameServerInfo>& gameServerInfoList);
	void OnMetaServerError(int errorcause, std::string errorMessage);


	void OnReceiveGameInfo(GameInitSettings gameInitSettings, ChangeEventList changeEventList);

	std::list<GameServerInfo> LANGameList;
	std::list<GameServerInfo> InternetGameList;

	StaticContainer WindowWidget;

	VBox			MainVBox;
	HBox			MainHBox;

    Label           CaptionLabel;

	HBox			ConnectHBox;
	TextBox			TextBox_ConnectHost;
	TextBox			TextBox_ConnectPort;
	TextButton		Button_Connect;

    // left VBox with create game buttons
    VBox            LeftVBox;
    TextButton		Button_CreateLANGame;
	TextButton		Button_CreateInternetGame;

    // right VBox with game list
    VBox            RightVBox;
    HBox            GameTypeButtonsHBox;
    TextButton      LANGamesButton;
    TextButton      InternetGamesButton;
    ListBox         GameList;

    // bottom row of buttons
    HBox            ButtonHBox;
    TextButton		Button_Join;
	TextButton		Button_Back;
};

#endif // MULTIPLAYERMENU_H

