/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef OPTIONSMENU_H
#define OPTIONSMENU_H

#include "MenuBase.h"
#include <GUI/StaticContainer.h>
#include <GUI/HBox.h>
#include <GUI/VBox.h>
#include <GUI/TextButton.h>
#include <GUI/TextBox.h>
#include <GUI/Checkbox.h>
#include <GUI/DropDownBox.h>
#include <DataTypes.h>

#include <SDL.h>
#include <vector>


#define MENU_QUIT_REINITIALIZE  (1)


class OptionsMenu : public MenuBase
{
public:
	OptionsMenu();
	virtual ~OptionsMenu();

private:
	void	OnChangeOption(bool bInteractive);
	void	OnOptionsOK();
	void	OnOptionsCancel();
	void    OnGameOptions();

	void	SaveConfiguration2File();

	virtual void OnChildWindowClose(Window* pChildWindow);

	/**
        This method determines all available suitable screen resolutions.
        If the system returns that any resolution is possible a reasonable
        set of resolutions is provided.
    */
    void determineAvailableScreenResolutions();

	std::vector<Coord> availScreenRes;

    SettingsClass::GameOptionsClass currentGameOptions;

	StaticContainer	WindowWidget;

	VBox		MainVBox;

	HBox        NameHBox;
	TextBox		TextBox_Name;
	HBox        GameOptionsHBox;
	TextButton  GameOptionsButton;

	HBox        LanguageHBox;
	DropDownBox DropDownBox_Language;
	HBox        GeneralHBox;
	Checkbox	Checkbox_Intro;

	HBox        ResolutionHBox;
	DropDownBox DropDownBox_Resolution;
	DropDownBox DropDownBox_Zoomlevel;
	HBox        VideoHBox;
	Checkbox	Checkbox_FullScreen;
	Checkbox	Checkbox_DoubleBuffering;

	HBox        AudioHBox;
	Checkbox    Checkbox_PlaySFX;
	Checkbox    Checkbox_PlayMusic;

	HBox        NetworkPortHBox;
    TextBox     TextBox_Port;
    HBox        NetworkMetaServerHBox;
    TextBox     TextBox_MetaServer;

	HBox		OkCancel_HBox;
	TextButton	Button_Accept;
	TextButton	Button_Back;
};


#endif // OPTIONSMENU_H
