/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PLAYER_H
#define PLAYER_H

#include <DataTypes.h>
#include <misc/InputStream.h>
#include <misc/OutputStream.h>

class House;
class ObjectBase;

class Player {
public:
    Player(House* associatedHouse, std::string playername);
    Player(InputStream& stream, House* associatedHouse);
    virtual ~Player();
    virtual void save(OutputStream& stream) const;

    virtual void update() = 0;

    virtual void onIncrementStructures(int itemID) = 0;
    virtual void onDecrementStructures(int itemID, const Coord& location) = 0;
    virtual void onDamage(ObjectBase* pObject, const Coord& location) = 0;

    House* getHouse() const { return pHouse; };
    Uint8 getPlayerID() const { return playerID; };

    std::string getPlayername() const { return playername; };
    void setPlayername(std::string playername) { this->playername = playername; };

private:
    friend class House;

    House* pHouse;
    Uint8 playerID;
    std::string playername;
};

#endif // PLAYER_H
