/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <FileClasses/BriefingText.h>

#include <misc/string_util.h>

#include <SDL_endian.h>

BriefingText::BriefingText(SDL_RWops* RWop) {
	if(RWop == NULL) {
	    throw std::invalid_argument("BriefingText: RWop == NULL!");
	}

	int BriefingTextFilesize = SDL_RWseek(RWop,0,SEEK_END);
	if(BriefingTextFilesize <= 0) {
	    throw std::runtime_error("BriefingText: SDL_RWseek() failed!");
	}

	if(BriefingTextFilesize < 2) {
	    throw std::runtime_error("BriefingText: No valid Briefing-File: File too small!");
	}

	if(SDL_RWseek(RWop,0,SEEK_SET) != 0) {
		throw std::runtime_error("BriefingText: SDL_RWseek() failed!");
	}

	uint8_t* pFiledata = new uint8_t[BriefingTextFilesize];

	if(SDL_RWread(RWop, pFiledata, BriefingTextFilesize, 1) != 1) {
	    delete [] pFiledata;
        throw std::runtime_error("BriefingText: SDL_RWread() failed!");
	}

	uint16_t numBriefingStrings = (SDL_SwapLE16( ((uint16_t*) pFiledata)[0] ) ) / sizeof(uint16_t) - 1;

	if(numBriefingStrings*sizeof(uint16_t) > (uint16_t) BriefingTextFilesize) {
	    delete [] pFiledata;
        throw std::runtime_error("BriefingText: No valid Briefing-File: File is smaller than specified number of briefing texts!");
	}

	uint16_t* Index = (uint16_t*) pFiledata;

	for(unsigned int i=0; i <= numBriefingStrings; i++) {
		Index[i] = SDL_SwapLE16(Index[i]);
	}

    try {
        for(unsigned int i=0; i < numBriefingStrings;i++) {
            std::string tmp = (const char*) (pFiledata + Index[i]);
            BriefingStrings.push_back(convertCP850ToISO8859_1(decodeString(tmp)));
        }
    } catch (std::exception &e) {
        delete [] pFiledata;
        throw;
    }

	delete [] pFiledata;
}

BriefingText::~BriefingText() {
}
