/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <FileClasses/TextManager.h>

#include <globals.h>

#include <FileClasses/FileManager.h>


TextManager::TextManager() {
    // load all briefing texts
	SDL_RWops* text_lng[3];
	text_lng[0] = pFileManager->OpenFile("TEXTA." + settings.General.LanguageExt);
	text_lng[1] = pFileManager->OpenFile("TEXTO." + settings.General.LanguageExt);
	text_lng[2] = pFileManager->OpenFile("TEXTH." + settings.General.LanguageExt);

	for(int i=0;i<3;i++) {
		if(text_lng[i] == NULL) {
			fprintf(stderr,"TextManager::TextManager: Can not open language file\n");
			exit(EXIT_FAILURE);
		}
		BriefingStrings[i] = std::shared_ptr<BriefingText>(new BriefingText(text_lng[i]));
		SDL_RWclose(text_lng[i]);
	}


	// load dune text
    SDL_RWops* dune_lng = pFileManager->OpenFile("DUNE." + settings.General.LanguageExt);
	pDuneText = std::shared_ptr<IndexedTextFile>(new IndexedTextFile(dune_lng));
	SDL_RWclose(dune_lng);

    // load all mentat texts
	SDL_RWops* mentat_lng[3];
	mentat_lng[0] = pFileManager->OpenFile("MENTATA." + settings.General.LanguageExt);
	mentat_lng[1] = pFileManager->OpenFile("MENTATO." + settings.General.LanguageExt);
	mentat_lng[2] = pFileManager->OpenFile("MENTATH." + settings.General.LanguageExt);

	for(int i=0;i<3;i++) {
		if(mentat_lng[i] == NULL) {
			fprintf(stderr,"TextManager::TextManager: Can not open mentat language file\n");
			exit(EXIT_FAILURE);
		}
		MentatStrings[i] = std::shared_ptr<MentatTextFile>(new MentatTextFile(mentat_lng[i]));
		SDL_RWclose(mentat_lng[i]);
	}


	// init languages
	switch(settings.General.Language) {
        case LNG_FRE:
            Init_Localization_French();
            break;
        case LNG_GER:
            Init_Localization_German();
            break;
        case LNG_ENG:
        default:
            // nothing
            break;
	}
}

TextManager::~TextManager() {
}

std::string	TextManager::getBriefingText(unsigned int mission, unsigned int texttype, int house) const {
	switch(house) {
		case HOUSE_ATREIDES:
		case HOUSE_FREMEN:
			return BriefingStrings[0]->getMissionBriefingString(mission,texttype);
			break;
		case HOUSE_ORDOS:
		case HOUSE_MERCENARY:
			return BriefingStrings[1]->getMissionBriefingString(mission,texttype);
			break;
		case HOUSE_HARKONNEN:
		case HOUSE_SARDAUKAR:
		default:
			return BriefingStrings[2]->getMissionBriefingString(mission,texttype);
			break;
    }
}

std::vector<MentatTextFile::MentatEntry> TextManager::getAllMentatEntries(int house, unsigned int techLevel) const {
    std::vector<MentatTextFile::MentatEntry> mentatEntries;

    switch(house) {
        case HOUSE_ATREIDES:
        case HOUSE_FREMEN: {
            for(unsigned int i = 0; i <  MentatStrings[0]->getNumEntries(); i++) {
                if(MentatStrings[0]->getMentatEntry(i).techLevel <= techLevel) {
                    mentatEntries.push_back(MentatStrings[0]->getMentatEntry(i));
                }
            }
        } break;
        case HOUSE_ORDOS:
        case HOUSE_MERCENARY: {
            for(unsigned int i = 0; i <  MentatStrings[1]->getNumEntries(); i++) {
                if(MentatStrings[1]->getMentatEntry(i).techLevel <= techLevel) {
                    mentatEntries.push_back(MentatStrings[1]->getMentatEntry(i));
                }
            }
        } break;
        case HOUSE_HARKONNEN:
        case HOUSE_SARDAUKAR:
        default: {
            for(unsigned int i = 0; i <  MentatStrings[2]->getNumEntries(); i++) {
                if(MentatStrings[2]->getMentatEntry(i).techLevel <= techLevel) {
                    mentatEntries.push_back(MentatStrings[2]->getMentatEntry(i));
                }
            }
        } break;
    }

    return mentatEntries;
}

void TextManager::Init_Localization_French() {

    // General strings
    localizedString["OK"] = "OK";
    localizedString["Back"] = "Retour";
    localizedString["BACK"] = "RETOUR";
    localizedString["Cancel"] = "Annuler";
    localizedString["CANCEL"] = "ANNULER";
    localizedString["Next"] = "Suivant";
    localizedString["NEXT"] = "SUIVANT";
    localizedString["Yes"] = "Oui";
    localizedString["No"] = "Non";
    localizedString["LOAD GAME"] = "CHARGER JEU";
    localizedString["Load Game"] = "Charger jeu";
    localizedString["SAVE GAME"] = "SAUVEGARDER JEU";
    localizedString["Save Game"] = "Sauvegarder jeu";
    localizedString["LOAD REPLAY"] = "CHARGER REPLAY";
    localizedString["Load Replay"] = "Charger Replay";

    // Main Menu
    localizedString["SINGLE PLAYER"] = "JEU SOLO";
    localizedString["MULTIPLAYER"] = "MULTIJOUEUR";
    localizedString["OPTIONS"] = "OPTIONS";
    localizedString["ABOUT"] = "ABOUT";
    localizedString["QUIT"] = "QUITTER";

    // Options Menu
    localizedString["Player Name"] = "Nom de joueur";
    localizedString["Concrete required"] = "B\351ton n\351cessaires";
    localizedString["Fog of War"] = "Brouillard de guerre";
    localizedString["Language"] = "Langue";
    localizedString["Play Intro"] = "Voir l'Introduction";
    localizedString["Video Resolution"] = "D\351finition d'\351cran";
    localizedString["Full Screen"] = "Plein \351cran";
    localizedString["Double Buffering"] = "Double Buffering";
    localizedString["Play SFX"] = "Jouer son effet";
    localizedString["Play Music"] = "Jouer musique";
    localizedString["Accept"] = "Accepter";

    // Single Player Menu
    localizedString["CAMPAIGN"] = "CAMPAGNE";
    localizedString["CUSTOM GAME"] = "JEU PERSONNALIS\311";
    localizedString["SKIRMISH"] = "ESCARMOUCHE";

    // Multi Player Menu
    localizedString["Connect"] = "Communiquer";
    localizedString["Create LAN Game"] = "Créer LAN-Jeu";
    localizedString["Create Internet Game"] = "Créer Internet-Jeu";
    localizedString["LAN Games"] = "LAN-jeux";
    localizedString["Internet Games"] = "Internet-jeux";
    localizedString["Join"] = "Rejoindre";
//    localizedString["The game host quit the game!"] = "";
//    localizedString["The server timed out!"] = "";
//    localizedString["A player with the same name as yours already exists!"] = "";
//    localizedString["There is no free player slot in this game left!"] = "";
//    localizedString["The connection to the game host was lost!"] = "";

    // Single Player Skirmish Menu
    localizedString["Start"] = "D\351marrer";

    // Custom Game Menu and Custom Player Menu
    localizedString["Custom Game"] = "Jeu personnalis\351";
    localizedString["LAN Game"] = "Multijoueur (LAN)";
    localizedString["Internet Game"] = "Multijoueur (Internet)";
    localizedString["SP Maps"] = "SP cartes";
    localizedString["SP User Maps"] = "propres SP c.";
    localizedString["MP Maps"] = "MP cartes";
    localizedString["MP User Maps"] = "propres MP c.";
    localizedString["Choose map"] = "choisir la carte";
    localizedString["No map available"] = "Carte pas dispo.";
    localizedString["Size"] = "Dimension";
    localizedString["Players"] = "Joueurs";
    localizedString["Author"] = "Auteur";
    localizedString["License"] = "Licence";
    localizedString["Multiple players per house"] = "Plusieurs joueurs par maison";
    localizedString["House"] = "Maison";
    localizedString["Team"] = "L'\351quipe";
    localizedString["Player"] = "Joueur";
    localizedString["Random"] = "Al\351atoire";
    localizedString["open"] = "ouverte";
    localizedString["closed"] = "ferm\351e";
    localizedString["At least 2 houses must be controlled\nby a human player or an AI player!"] = "Au moins 2 maisons doivent \352tre contr\364l\351es\npar un joueur humain ou un joueurs AI!";
    localizedString["There must be at least two different teams!"] = "Il doit y avoir au moins deux \351quipes diff\351rentes!";

    // Custom Game Stats Menu
//    localizedString["Built Object"] = "";
//    localizedString["Destroyed"] = "";
//    localizedString["Harvested Spice"] = "";

    // Load/Save Window
    localizedString["Load"] = "Charger";
    localizedString["Save"] = "Sauvegarder";

    // InGame Menu
    localizedString["Resume Game"] = "Reprendre la partie";
    localizedString["Game Settings"] = "Param\351tres de jeu";
    localizedString["Restart Game"] = "Recommencer le jeu";
    localizedString["Quit to Menu"] = "Retour au menu";
    localizedString["Do you really want to quit this game?"] = "Voulez-vous vraiment quitter ce jeu?";
    localizedString["Do you really want to restart this game?"] = "Voulez-vous vraiment \340 red\351marrer ce jeu?";

    // BuilderList
    localizedString["SOLD OUT"] = "EPUIS\311";
    localizedString["ALREADY\nBUILT"] = "D\331J\320 CONSTRUIT";
    localizedString["PLACE IT"] = "PLACER";
    localizedString["ON HOLD"] = "ATTENTE";
    localizedString["Order"]= "Commander";

    // Builder Interface
    localizedString["Upgrade"] = "Am\351liorer";

    // Unit Interface
    localizedString["Move to a position"] = "D\351placer vers une position";
    localizedString["Attack a unit, structure or position"] = "Attaque d'une unit\351, la structure ou la position";
    localizedString["Capture a building"] = "Capture d'un b\342timent";
    localizedString["Guard"] = "Garder";
    localizedString["Unit will not move from location"] = "Unit\351 ne se d\351place pas de l'endroit";
    localizedString["Area Guard"] = "Garder zone";
    localizedString["Unit will engage any unit within guard range"] = "Unit\351 engager tout appareil \340 port\351e de garde";
    localizedString["Stop"] = "Arr\352ter";
    localizedString["Unit will not move, nor attack"] = "Unit\351 ne se d\351place pas, ni d'attaque";
    localizedString["Ambush"] = "Embuscade";
    localizedString["Unit will not move until enemy unit spotted"] = "Unit\351 ne bougera pas jusqu'\340 unit\351 ennemie spoted";
    localizedString["Hunt"] = "Chasser";
    localizedString["Unit will immediately start to engage an enemy unit"] = "Unit\351 va imm\351diatement commencer \340 engager une unit\351 ennemie";
    localizedString["Deploy"] = "Deployer";
    localizedString["Build a new construction yard"] = "Construire un chantier de construction de nouvelles";
    localizedString["Destruct"] = "D\351truire";
    localizedString["Self-destruct this unit"] = "Auto d\351truire cette unit\351";

    // Windtrap Interface
    localizedString["Required"] = "n\351cess.";
    localizedString["Produced"] = "produit";

    // Refinery and Silo Interface
    localizedString["Capacity"] = "capac.";
    localizedString["Stored"] = "stock\351es";

    // Palace Interace
    localizedString["READY"] = "PR\312T";

    // Choam
    localizedString["New Starport prices"] = "Nouveau prix Starport";

    // Messages from Units and Structures
    localizedString["You cannot deploy here."] = "Vous ne pouvez pas d\351ployer ici.";
    localizedString["Not enough money"] = "Pas assez de cr\351dits.";
    localizedString["Palace is ready"] = "Palais est pr\352t.";
    localizedString["Unable to spawn Fremen"] = "Impossible pour frayer Fremen.";

    // Game Class
    localizedString["Screenshot saved"] = "Screenshot sauvegard\351es";
    localizedString["Game speed"] = "Vitesse de jeu";
    localizedString["You Have Completed Your Mission."] = "Vous avez accompli votre mission.";
    localizedString["You Have Failed Your Mission."] = "Vous avez \351chou\351 dans votre mission.";

    // House
    localizedString["House '%s' has been defeated."] = "Maison '%s' a \351t\351 d\351fait.";

}

void TextManager::Init_Localization_German() {

    // General strings
    localizedString["OK"] = "OK";
    localizedString["Back"] = "Zur\374ck";
    localizedString["BACK"] = "ZUR\334CK";
    localizedString["Cancel"] = "Abbrechen";
    localizedString["CANCEL"] = "ABBRECHEN";
    localizedString["Next"] = "Weiter";
    localizedString["NEXT"] = "WEITER";
    localizedString["Yes"] = "Ja";
    localizedString["No"] = "Nein";
    localizedString["LOAD GAME"] = "SPIEL LADEN";
    localizedString["Load Game"] = "Spiel laden";
    localizedString["SAVE GAME"] = "SPIEL SPEICHERN";
    localizedString["Save Game"] = "Spiel speichern";
    localizedString["LOAD REPLAY"] = "REPLAY LADEN";
    localizedString["Load Replay"] = "Replay laden";
    localizedString["Remove"] = "Entfernen";

    // Main Menu
    localizedString["SINGLE PLAYER"] = "EINZELSPIELER";
    localizedString["MULTIPLAYER"] = "MEHRSPIELER";
    localizedString["OPTIONS"] = "OPTIONEN";
    localizedString["ABOUT"] = "\334BER";
    localizedString["QUIT"] = "BEENDEN";

    // Options Menu
    localizedString["Game Options"] = "Spieleinstellungen";
    localizedString["Game Options..."] = "Spieleinstellungen...";
    localizedString["Default Game Options"] = "Std. Spieleinstellungen";
    localizedString["Change..."] = "\304ndern...";
    localizedString["Player Name"] = "Spielername";
    localizedString["Please enter a Player Name."] = "Bitte einen Spielernamen eingeben.";
    localizedString["Concrete Required"] = "Betonplatten ben\366tigt";
    localizedString["If checked building on bare rock will result in 50% structure health penalty."] = "Falls gew\344hlt werden auf blankem Felsen gebaute Geb\344ude mit nur 50% Geb\344udezustand errichtet.";
    localizedString["Structures Degrade On Concrete"] = "Geb\344ude gehen auch auf Betonplatten kaputt";
    localizedString["If checked structures will degrade on power shortage even if built on concrete."] = "Falls gew\344hlt werden Geb\344ude auch dann von Stromausf\344llen in Mitleidenschaft gezogen, wenn sie auf Beton gebaut sind.";
    localizedString["Fog of War"] = "Nebel des Krieges";
    localizedString["If checked explored terrain will become foggy when no unit or structure is next to it."] = "Falls gew\344hlt werden Gebiete vernebelt, wenn keine Einheit oder Geb\344ude in der N\344he ist.";
    localizedString["Start with Explored Map"] = "Mit aufgedeckter Karte starten";
    localizedString["If checked the complete map is unhidden at the beginning of the game."] = "Falls gew\344hlt wird die komplette Karte am Anfang des Spiels aufgedeckt.";
    localizedString["Instant Build"] = "Sofortiges Bauen";
    localizedString["If checked the building of structures and units does not take any time."] = "Falls gew\344hlt kostet der Bau von Geb\344uden und Einheiten keine Zeit.";
    localizedString["Language"] = "Sprache";
    localizedString["Play Intro"] = "Intro abspielen";
    localizedString["Video Resolution"] = "Bildschirmaufl\366sung";
    localizedString["Full Screen"] = "Vollbild";
    localizedString["Double Buffering"] = "Double Buffering";
    localizedString["Play SFX"] = "Soundeffekte";
    localizedString["Play Music"] = "Musik";
    localizedString["Port"] = "Port";
    localizedString["Server Port must be between 1 and 65535!\nDefault Server Port is %d!"] = "Server Port muss zwischen 1 und 65535 sein!\nStandardeinstellung ist Server Port %d.";
    localizedString["MetaServer"] = "MetaServer";
    localizedString["Please enter a MetaServer."] = "Bitte einen MetaServer eingeben.";

    localizedString["Accept"] = "Annehmen";

    // Single Player Menu
    localizedString["CAMPAIGN"] = "KAMPAGNE";
    localizedString["CUSTOM GAME"] = "FREIES SPIEL";
    localizedString["SKIRMISH"] = "GEPL\304NKEL";

    // Multi Player Menu
    localizedString["Connect"] = "Verbinden";
    localizedString["Create LAN Game"] = "LAN-Spiel erstellen";
    localizedString["Create Internet Game"] = "Internet-Spiel erstellen";
    localizedString["LAN Games"] = "LAN-Spiele";
    localizedString["Internet Games"] = "Internet-Spiele";
    localizedString["Join"] = "Beitreten";
    localizedString["The game host quit the game!"] = "Der Ersteller des Spiels hat das Spiel verlassen!";
    localizedString["The server timed out!"] = "Die Verbindung zum Server hat das timeout \374berschritten!";
    localizedString["A player with the same name as yours already exists!"] = "Es existiert bereits ein Spieler mit Ihrem Namen!";
    localizedString["There is no free player slot in this game left!"] = "Es gibt keine weiteren freien Pl\344tze in diesem Spiel!";
    localizedString["The connection to the game host was lost!"] = "Die Verbindung zum Ersteller des Spiels ist abgebrochen!";

    // Single Player Skirmish Menu
    localizedString["Start"] = "Starten";

    // Custom Game Menu and Custom Player Menu
    localizedString["Custom Game"] = "Freies Spiel";
    localizedString["LAN Game"] = "LAN-Spiel";
    localizedString["Internet Game"] = "Internet-Spiel";
    localizedString["SP Maps"] = "SP Karten";
    localizedString["SP User Maps"] = "eigene SP K.";
    localizedString["MP Maps"] = "MP Karten";
    localizedString["MP User Maps"] = "eigene MP K.";
    localizedString["Choose map"] = "Karte w\344hlen";
    localizedString["No map available"] = "Karte nicht verf\374g.";
    localizedString["Size"] = "Gr\366\337e";
    localizedString["Players"] = "Spieler";
    localizedString["Author"] = "Ersteller";
    localizedString["License"] = "Lizenz";
    localizedString["Multiple players per house"] = "Mehrere Spieler pro Haus";
    localizedString["House"] = "Haus";
    localizedString["Team"] = "Team";
    localizedString["Player"] = "Spieler";
    localizedString["Random"] = "Zuf\344llig";
    localizedString["open"] = "offen";
    localizedString["closed"] = "geschlossen";
    localizedString["At least 2 houses must be controlled\nby a human player or an AI player!"] = "Mindestens zwei H\344user m\374ssen von einem menschlichen\nSpieler oder dem Computer gespielt werden!";
    localizedString["There must be at least two different teams!"] = "Es muss mindestens zwei verschiedene Teams geben!";

    // Custom Game Stats Menu
    localizedString["Built Object"] = "Gebaute Objekte";
    localizedString["Destroyed"] = "Zerst\366rt";
    localizedString["Harvested Spice"] = "Gew\374rz";

    // Load/Save Window
    localizedString["Load"] = "Laden";
    localizedString["Save"] = "Speichern";

    // InGame Menu
    localizedString["Resume Game"] = "Wiederaufnehmen";
    localizedString["Game Settings"] = "Spieleinstellungen";
    localizedString["Restart Game"] = "Spiel neustarten";
    localizedString["Quit to Menu"] = "Zur\374ck zum Men\374";
    localizedString["Invalid file name! File names must not contain \\ or / and must not be empty!"] = "Ung\374ltiger Dateiname. Dateinamen d\374rfen kein \\ oder / enthalten und nicht leer sein!";
    localizedString["Do you really want to delete '%s' ?"] = "M\366chten Sie wirklich '%s' l\366schen?";
    localizedString["Do you really want to quit this game?"] = "Dieses Spiel wirklich beenden?";
    localizedString["Do you really want to restart this game?"] = "Dieses Spiel wirklich neustarten?";

    // BuilderList
    localizedString["SOLD OUT"] = "AUS-\nVERKAUFT";
    localizedString["ALREADY\nBUILT"] = "BEREITS\nGEBAUT";
    localizedString["PLACE IT"] = "PLAZIEREN";
    localizedString["ON HOLD"] = "GESTOPPT";
    localizedString["Order"]= "Bestellen";

    // Builder Interface
    localizedString["Upgrade"] = "Verbessern";

    // Unit Interface
    localizedString["Move to a position"] = "Zu einer Position bewegen";
    localizedString["Attack a unit, structure or position"] = "Eine Einheit, Geb\344ude oder Position angreifen";
    localizedString["Capture a building"] = "Ein Geb\344ude einnehmen";
    localizedString["Guard"] = "Bewachen";
    localizedString["Unit will not move from location"] = "Die Einheit wird sich von ihrem Ort nicht wegbewegen";
    localizedString["Area Guard"] = "\334berwachen";
    localizedString["Unit will engage any unit within guard range"] = "Jede feindliche Einheit in der Umgebung verfolgen";
    localizedString["Stop"] = "Stoppen";
    localizedString["Unit will not move, nor attack"] = "Die Einheit wird sich weder bewegen noch angreifen";
    localizedString["Ambush"] = "Hinterhalt";
    localizedString["Unit will not move until enemy unit spotted"] = "Die Einheit bewegt sich erst, wenn sie einen Gegner sieht";
    localizedString["Hunt"] = "Jagd";
    localizedString["Unit will immediately start to engage an enemy unit"] = "Die Einheit sucht sich sofort einen Gegner";
    localizedString["Deploy"] = "Einsetzen";
    localizedString["Build a new construction yard"] = "Bauhof aufbauen";
    localizedString["Destruct"] = "Zerst\366ren";
    localizedString["Self-destruct this unit"] = "Einheit selbst-zerst\366ren";

    // Windtrap Interface
    localizedString["Required"] = "Ben\366t.";
    localizedString["Produced"] = "Prod.";

    // Refinery and Silo Interface
    localizedString["Capacity"] = "Kapaz.";
    localizedString["Stored"] = "Inhalt";

    // Palace Interace
    localizedString["READY"] = "BEREIT";

    // Choam
    localizedString["New Starport prices"] = "Neue Preise am Raumflughafen";

    // Messages from Units and Structures
    localizedString["You cannot deploy here."] = "Kann hier nicht eingesetzt werden.";
    localizedString["Not enough money"] = "Nicht gen\374gend Credits.";
    localizedString["Palace is ready"] = "Palast ist bereit.";
    localizedString["Unable to spawn Fremen"] = "Kann Fremen nicht rufen.";
    localizedString["Game paused!"] = "Spiel angehalten!";
    localizedString["Game resumed!"] = "Spiel fortgesetzt!";

    // Game
    localizedString["Screenshot saved"] = "Screenshot gespeichert";
    localizedString["Game speed"] = "Spielgeschwindigkeit";
    localizedString["You Have Completed Your Mission."] = "Mission erfolgreich abgeschlossen.";
    localizedString["You Have Failed Your Mission."] = "Mission gescheitert.";
    localizedString["Waiting for other players... "] = "Auf andere Spieler warten... ";

    // House
    localizedString["House '%s' has been defeated."] = "Haus '%s' wurde geschlagen.";

}
