/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <GUI/dune/GameOptionsWindow.h>

#include <globals.h>

#include <FileClasses/TextManager.h>


GameOptionsWindow::GameOptionsWindow(SettingsClass::GameOptionsClass& initialGameOptions) : Window(50,50,400,240)
{
    gameOptions = initialGameOptions;

    SetWindowWidget(&vbox);
    vbox.AddWidget(VSpacer::Create(6));

    captionlabel.SetText(pTextManager->getLocalized("Game Options"));
    vbox.AddWidget(&captionlabel);
    vbox.AddWidget(VSpacer::Create(3));
    vbox.AddWidget(&hbox);
    vbox.AddWidget(VSpacer::Create(6));
    hbox.AddWidget(Spacer::Create());
    hbox.AddWidget(&vbox2);
    vbox2.AddWidget(Spacer::Create());

    concreteRequiredCheckbox.SetText(pTextManager->getLocalized("Concrete Required"));
    concreteRequiredCheckbox.SetTooltipText(pTextManager->getLocalized("If checked building on bare rock will result in 50% structure health penalty."));
    concreteRequiredCheckbox.SetChecked(gameOptions.ConcreteRequired);
    vbox2.AddWidget(&concreteRequiredCheckbox);
    vbox2.AddWidget(VSpacer::Create(4));

    structuresDegradeOnConcreteCheckbox.SetText(pTextManager->getLocalized("Structures Degrade On Concrete"));
    structuresDegradeOnConcreteCheckbox.SetTooltipText(pTextManager->getLocalized("If checked structures will degrade on power shortage even if built on concrete."));
    structuresDegradeOnConcreteCheckbox.SetChecked(gameOptions.StructuresDegradeOnConcrete);
    vbox2.AddWidget(&structuresDegradeOnConcreteCheckbox);
    vbox2.AddWidget(VSpacer::Create(4));

    fogOfWarCheckbox.SetText(pTextManager->getLocalized("Fog of War"));
    fogOfWarCheckbox.SetTooltipText(pTextManager->getLocalized("If checked explored terrain will become foggy when no unit or structure is next to it."));
    fogOfWarCheckbox.SetChecked(gameOptions.FogOfWar);
    vbox2.AddWidget(&fogOfWarCheckbox);
    vbox2.AddWidget(VSpacer::Create(4));

    startWithExploredMapCheckbox.SetText(pTextManager->getLocalized("Start with Explored Map"));
    startWithExploredMapCheckbox.SetTooltipText(pTextManager->getLocalized("If checked the complete map is unhidden at the beginning of the game."));
    startWithExploredMapCheckbox.SetChecked(gameOptions.StartWithExploredMap);
    vbox2.AddWidget(&startWithExploredMapCheckbox);
    vbox2.AddWidget(VSpacer::Create(4));

    instantBuildCheckbox.SetText(pTextManager->getLocalized("Instant Build"));
    instantBuildCheckbox.SetTooltipText(pTextManager->getLocalized("If checked the building of structures and units does not take any time."));
    instantBuildCheckbox.SetChecked(gameOptions.InstantBuild);
    vbox2.AddWidget(&instantBuildCheckbox);
    vbox2.AddWidget(VSpacer::Create(4));


    vbox2.AddWidget(Spacer::Create());

    vbox2.AddWidget(VSpacer::Create(4));

    okbutton.SetText(pTextManager->getLocalized("OK"));
    okbutton.SetOnClick(std::bind(&GameOptionsWindow::OnOK, this));
    vbox2.AddWidget(&okbutton, 30);
    vbox2.AddWidget(VSpacer::Create(4));
    hbox.AddWidget(Spacer::Create());
    captionlabel.SetAlignment(Alignment_HCenter);

    int xpos = std::max(0,(screen->w - GetSize().x)/2);
	int ypos = std::max(0,(screen->h - GetSize().y)/2);

	SetCurrentPosition(xpos,ypos,GetSize().x,GetSize().y);
}

GameOptionsWindow::~GameOptionsWindow()
{
	;
}

void GameOptionsWindow::OnOK()
{
    gameOptions.ConcreteRequired = concreteRequiredCheckbox.IsChecked();
    gameOptions.StructuresDegradeOnConcrete = structuresDegradeOnConcreteCheckbox.IsChecked();
    gameOptions.FogOfWar = fogOfWarCheckbox.IsChecked();
    gameOptions.StartWithExploredMap = startWithExploredMapCheckbox.IsChecked();
    gameOptions.InstantBuild = instantBuildCheckbox.IsChecked();

    Window* pParentWindow = dynamic_cast<Window*>(GetParent());
    if(pParentWindow != NULL) {
        pParentWindow->CloseChildWindow();
    }
}
