/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <GUI/dune/WaitingForOtherPlayers.h>

#include <globals.h>

#include <FileClasses/TextManager.h>
#include <players/HumanPlayer.h>
#include <Network/NetworkManager.h>

#include <Game.h>

#include <list>
#include <string>


WaitingForOtherPlayers::WaitingForOtherPlayers() : Window(50,50,50,50)
{
    SetWindowWidget(&vbox);
    vbox.AddWidget(VSpacer::Create(6));

    vbox.AddWidget(&textlabel);
    vbox.AddWidget(VSpacer::Create(3));
    vbox.AddWidget(&hbox);
    vbox.AddWidget(VSpacer::Create(6));
    hbox.AddWidget(Spacer::Create());
    hbox.AddWidget(&vbox2);
    vbox2.AddWidget(VSpacer::Create(4));
    removebutton.SetText(pTextManager->getLocalized("Remove"));
    removebutton.SetOnClick(std::bind(&WaitingForOtherPlayers::OnRemove, this));
    removebutton.SetVisible(false);
    removebutton.SetEnabled(false);
    vbox2.AddWidget(&removebutton);
    vbox2.AddWidget(VSpacer::Create(4));
    hbox.AddWidget(Spacer::Create());
    textlabel.SetAlignment(Alignment_HCenter);

    update();

    int xpos = std::max(0,(screen->w - GetSize().x)/2);
	int ypos = std::max(0,(screen->h - GetSize().y)/2);

	SetCurrentPosition(xpos,ypos,GetSize().x,GetSize().y);
}

WaitingForOtherPlayers::~WaitingForOtherPlayers()
{
	;
}

void WaitingForOtherPlayers::update() {
    std::string text = pTextManager->getLocalized("Waiting for other players... ");

    // test if we need to wait for data to arrive
    std::list<std::string> peerList = pNetworkManager->getConnectedPeers();
    std::list<std::string>::iterator iter;
    for(iter = peerList.begin(); iter != peerList.end(); ++iter) {
        HumanPlayer* pPlayer = dynamic_cast<HumanPlayer*>(currentGame->getPlayerByName(*iter));
        if(pPlayer != NULL) {
            if(pPlayer->nextExpectedCommandsCycle <= currentGame->GetGameCycleCount()) {
                text += "\n" + pPlayer->getPlayername();
            }
        }
    }

    SetText(text);
}

void WaitingForOtherPlayers::OnRemove()
{
	currentGame->ResumeGame();
}
