/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <Menu/CustomGameStatsMenu.h>

#include <globals.h>

#include <misc/string_util.h>
#include <misc/FileSystem.h>

#include <FileClasses/GFXManager.h>
#include <FileClasses/TextManager.h>

#include <Game.h>
#include <House.h>
#include <sand.h>

#include <algorithm>


CustomGameStatsMenu::CustomGameStatsMenu() : MenuBase()
{
	// set up window
	SDL_Surface *surf;
	surf = pGFXManager->getUIGraphic(UI_MenuBackground);

	SetBackground(surf,false);
	Resize(surf->w,surf->h);

	SetWindowWidget(&WindowWidget);

	int localHouseColor = houseColor[pLocalHouse->getHouseID()];

	WindowWidget.AddWidget(&MainVBox, Point(24,23),	Point(screen->w - 48, screen->h - 32));

    CaptionLabel.SetText(getBasename(currentGame->getGameInitSettings().getFilename(), true));
    CaptionLabel.SetTextColor(localHouseColor + 3);
    CaptionLabel.SetAlignment(Alignment_HCenter);
    MainVBox.AddWidget(&CaptionLabel, 24);
    MainVBox.AddWidget(VSpacer::Create(24));

    MainVBox.AddWidget(Spacer::Create(), 0.05);

    MainVBox.AddWidget(&MainHBox, 0.80);

    MainHBox.AddWidget(Spacer::Create(), 0.4);
    MainHBox.AddWidget(&PlayerStatListVBox, 0.2);

    HeaderHBox.AddWidget(&HeaderLabelDummy, 130);
    HeaderHBox.AddWidget(Spacer::Create(), 10);
    HeaderLabel1.SetText(pTextManager->getLocalized("Built Object"));
    HeaderLabel1.SetAlignment(Alignment_HCenter);
    HeaderLabel1.SetTextColor(localHouseColor + 3);
    HeaderHBox.AddWidget(&HeaderLabel1, 132);
    HeaderHBox.AddWidget(Spacer::Create(), 30);
    HeaderLabel2.SetText(pTextManager->getLocalized("Destroyed"));
    HeaderLabel2.SetAlignment(Alignment_HCenter);
    HeaderLabel2.SetTextColor(localHouseColor + 3);
    HeaderHBox.AddWidget(&HeaderLabel2, 132);
    HeaderLabel3.SetText(pTextManager->getLocalized("Harvested Spice"));
    HeaderLabel3.SetAlignment(Alignment_HCenter);
    HeaderLabel3.SetTextColor(localHouseColor + 3);
    HeaderHBox.AddWidget(Spacer::Create(), 30);
    HeaderHBox.AddWidget(&HeaderLabel3, 132);

    PlayerStatListVBox.AddWidget(&HeaderHBox, 25);

    PlayerStatListVBox.AddWidget(VSpacer::Create(15));

    int maxBuiltValue = 0;
    int maxDestroyedValue = 0;
    float maxSpiceHarvested = 0.0;

    for(int i=0;i<NUM_HOUSES;i++) {
        House* pHouse = currentGame->house[i];

        if(pHouse != NULL) {
            maxBuiltValue = std::max(maxBuiltValue, pHouse->getBuiltValue());
            maxDestroyedValue = std::max(maxDestroyedValue, pHouse->getDestroyedValue());
            maxSpiceHarvested = std::max(maxSpiceHarvested, pHouse->getHarvestedSpice());
        }
    }

    for(int i=0;i<NUM_HOUSES;i++) {
        HouseStat& curHouseStat = houseStat[i];
        House* pHouse = currentGame->house[i];

        if(pHouse != NULL) {
            int color = houseColor[i];

            curHouseStat.HouseName.SetText(pTextManager->getLocalized("House") + " " + getHouseNameByNumber((HOUSETYPE) i));
            curHouseStat.HouseName.SetTextColor(color + 3);
            curHouseStat.HouseHBox.AddWidget(&curHouseStat.HouseName, 130);
            curHouseStat.HouseHBox.AddWidget(Spacer::Create(), 10);

            curHouseStat.Value1.SetText( stringify(pHouse->getBuiltValue()*100));
            curHouseStat.Value1.SetTextFont(FONT_STD10);
            curHouseStat.Value1.SetAlignment(Alignment_Right);
            curHouseStat.Value1.SetTextColor(color + 3);
            curHouseStat.HouseHBox.AddWidget(&curHouseStat.Value1, 50);
            curHouseStat.HouseHBox.AddWidget(HSpacer::Create(2));
            curHouseStat.ProgressBar1.SetProgress( (maxBuiltValue == 0) ? 0.0 : (pHouse->getBuiltValue() * 100.0f / maxBuiltValue));
            curHouseStat.ProgressBar1.SetDrawShadow(true);
            curHouseStat.ProgressBar1.SetColor(color + 1);
            curHouseStat.VBox1.AddWidget(Spacer::Create(), 0.5);
            curHouseStat.VBox1.AddWidget(&curHouseStat.ProgressBar1, 12);
            curHouseStat.VBox1.AddWidget(Spacer::Create(), 0.5);
            curHouseStat.HouseHBox.AddWidget(&curHouseStat.VBox1, 80);

            curHouseStat.HouseHBox.AddWidget(Spacer::Create(), 30);

            curHouseStat.Value2.SetText( stringify(pHouse->getDestroyedValue()*100));
            curHouseStat.Value2.SetTextFont(FONT_STD10);
            curHouseStat.Value2.SetAlignment(Alignment_Right);
            curHouseStat.Value2.SetTextColor(color + 3);
            curHouseStat.HouseHBox.AddWidget(&curHouseStat.Value2, 50);
            curHouseStat.HouseHBox.AddWidget(HSpacer::Create(2));
            curHouseStat.ProgressBar2.SetProgress( (maxDestroyedValue == 0) ? 0.0 : (pHouse->getDestroyedValue() * 100.0f / maxDestroyedValue));
            curHouseStat.ProgressBar2.SetDrawShadow(true);
            curHouseStat.ProgressBar2.SetColor(color + 1);
            curHouseStat.VBox2.AddWidget(Spacer::Create(), 0.5);
            curHouseStat.VBox2.AddWidget(&curHouseStat.ProgressBar2, 12);
            curHouseStat.VBox2.AddWidget(Spacer::Create(), 0.5);
            curHouseStat.HouseHBox.AddWidget(&curHouseStat.VBox2, 80);

            curHouseStat.HouseHBox.AddWidget(Spacer::Create(), 30);

            curHouseStat.Value3.SetText( stringify((int) pHouse->getHarvestedSpice()));
            curHouseStat.Value3.SetTextFont(FONT_STD10);
            curHouseStat.Value3.SetAlignment(Alignment_Right);
            curHouseStat.Value3.SetTextColor(color + 3);
            curHouseStat.HouseHBox.AddWidget(&curHouseStat.Value3, 50);
            curHouseStat.HouseHBox.AddWidget(HSpacer::Create(2));
            curHouseStat.ProgressBar3.SetProgress( (maxSpiceHarvested == 0.0) ? 0.0 : (pHouse->getHarvestedSpice() * 100.0f / maxSpiceHarvested));
            curHouseStat.ProgressBar3.SetDrawShadow(true);
            curHouseStat.ProgressBar3.SetColor(color + 1);
            curHouseStat.VBox3.AddWidget(Spacer::Create(), 0.5);
            curHouseStat.VBox3.AddWidget(&curHouseStat.ProgressBar3, 12);
            curHouseStat.VBox3.AddWidget(Spacer::Create(), 0.5);
            curHouseStat.HouseHBox.AddWidget(&curHouseStat.VBox3, 80);

            PlayerStatListVBox.AddWidget(&curHouseStat.HouseHBox, 20);

            PlayerStatListVBox.AddWidget(VSpacer::Create(15));
        }
    }

    MainHBox.AddWidget(Spacer::Create(), 0.4);

    MainVBox.AddWidget(Spacer::Create(), 0.05);

    MainVBox.AddWidget(VSpacer::Create(20));
	MainVBox.AddWidget(&ButtonHBox, 24);
	MainVBox.AddWidget(VSpacer::Create(14), 0.0);

    ButtonHBox.AddWidget(HSpacer::Create(70));
    int totalTime = currentGame->GetGameTime()/1000;
    TextTime.SetText(strprintf(pTextManager->getDuneText(DuneText_Time).c_str(), totalTime/3600, (totalTime%3600)/60));
    TextTime.SetTextColor(localHouseColor + 3);
	ButtonHBox.AddWidget(&TextTime, 0.2);

	ButtonHBox.AddWidget(Spacer::Create(), 0.0625);
	ButtonHBox.AddWidget(Spacer::Create(), 0.475);
	ButtonHBox.AddWidget(Spacer::Create(), 0.0625);

    Button_OK.SetText(pTextManager->getLocalized("OK"));
    Button_OK.SetTextColor(localHouseColor + 3);
	Button_OK.SetOnClick(std::bind(&CustomGameStatsMenu::OnOK, this));
	ButtonHBox.AddWidget(&Button_OK, 0.2);
	ButtonHBox.AddWidget(HSpacer::Create(90));
}

CustomGameStatsMenu::~CustomGameStatsMenu()
{
	;
}

void CustomGameStatsMenu::OnOK()
{
    quit();
}

