/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <Menu/MentatHelp.h>

#include <globals.h>

#include <sand.h>

#include <FileClasses/GFXManager.h>
#include <FileClasses/TextManager.h>

#include <misc/draw_util.h>

#include <Game.h>

MentatHelp::MentatHelp(int newHouse, int techLevel, int mission) : MentatMenu(newHouse), mission(mission) {
    mentatEntries = pTextManager->getAllMentatEntries(newHouse, techLevel);

    if(mission == 0) {
        std::vector<MentatTextFile::MentatEntry>::iterator iter = mentatEntries.begin();
        while(iter != mentatEntries.end()) {
            if(iter->numMenuEntry == 0) {
                iter = mentatEntries.erase(iter);
            } else {
                ++iter;
            }
        }

    }

	SetClearScreen(false);

	BackgroundLabel.SetTextColor(-1, -1, 116);
	WindowWidget.AddWidget(&BackgroundLabel,Point(256,96),Point(368,224));

    std::vector<MentatTextFile::MentatEntry>::iterator iter;
    for(iter = mentatEntries.begin(); iter != mentatEntries.end(); ++iter) {
        if(iter->menuLevel == 0) {
            MentatTopicsList.AddEntry("     " + iter->title + " :");
        } else {
            MentatTopicsList.AddEntry("        " + iter->title);
        }
    }
    MentatTopicsList.setHighlightSelectedElement(false);
	MentatTopicsList.SetOnSingleClick(std::bind(&MentatHelp::OnListBoxClick, this));
	MentatTopicsList.SetColor(houseColor[newHouse]+3);
    WindowWidget.AddWidget(&MentatTopicsList,Point(256+7, 96+7),Point(368 - 14, 224 - 14));

    WindowWidget.AddWidget(&Anim,Point(256,96),Point(368,224));
    Anim.SetVisible(false);
    Anim.SetEnabled(false);
    ItemDescriptionLabel.SetTextFont(FONT_STD10);
    ItemDescriptionLabel.SetTextColor(COLOR_WHITE, COLOR_BLACK);
    WindowWidget.AddWidget(&ItemDescriptionLabel,Point(256 + 4, 96 + 4),Point(368 - 8, 224 - 8));
    ItemDescriptionLabel.SetVisible(false);

	SDL_Surface* surf = pGFXManager->getUIGraphic(UI_MentatExit);
	SDL_Surface* surfPressed = pGFXManager->getUIGraphic(UI_MentatExit_Pressed);
	ExitButton.SetSurfaces(surf,false,surfPressed,false);

	ExitButton.SetOnClick(std::bind(&MentatHelp::OnExit, this));
	WindowWidget.AddWidget(&ExitButton,Point(370,340),Point(surf->w,surf->h));
}

MentatHelp::~MentatHelp() {
}

void MentatHelp::DrawSpecificStuff()
{
    MentatMenu::DrawSpecificStuff();

    int x1 = GetPosition().x;
    int x2 = GetPosition().x + GetSize().x - 1;
    int y1 = GetPosition().y;
    int y2 = GetPosition().y + GetSize().y - 1;

	drawRect(screen, x1, y1, x2, y2, 229);
	drawHLine(screen, x1 + 1, y1 + 1, x2 - 1, 108);
	drawHLine(screen, x1 + 2, y1 + 2, x2 - 2, 108);
	drawHLine(screen, x1 + 3, y1 + 3, x2 - 3, 108);

	drawVLine(screen, x1 + 1, y1 + 1, y2 - 1, 108);
	drawVLine(screen, x1 + 2, y1 + 2, y2 - 2, 108);
	drawVLine(screen, x1 + 3, y1 + 3, y2 - 3, 108);

	drawHLine(screen, x1 + 1, y2 - 1, x2 - 1, 226);
	drawHLine(screen, x1 + 2, y2 - 2, x2 - 2, 226);
	drawHLine(screen, x1 + 3, y2 - 3, x2 - 3, 226);

	drawVLine(screen, x2 - 1, y1 + 1, y2 - 1, 226);
	drawVLine(screen, x2 - 2, y1 + 2, y2 - 2, 226);
	drawVLine(screen, x2 - 3, y1 + 3, y2 - 3, 226);
}

bool MentatHelp::doInput(SDL_Event &event)
{
    if((MentatTopicsList.IsVisible() == false) && (event.type == SDL_MOUSEBUTTONDOWN)) {
        showNextMentatText();
        return true;
    } else {
        return MentatMenu::doInput(event);
    }
}

void MentatHelp::onMentatTextFinished() {
    Anim.SetVisible(false);
    Anim.SetEnabled(false);
    ItemDescriptionLabel.SetVisible(false);
    MentatTopicsList.SetVisible(true);
    MentatTopicsList.SetEnabled(true);
    ExitButton.SetVisible(true);
    ExitButton.SetEnabled(true);

    setText("");
    ItemDescriptionLabel.SetText("");
}

void MentatHelp::OnExit() {
    if(MentatTopicsList.IsVisible()) {
        currentGame->ResumeGame();
        quit();
    } else {
        onMentatTextFinished();
    }
}

void MentatHelp::OnListBoxClick() {
    int index = MentatTopicsList.GetSelectedIndex();

    if(index < 0) {
        return;
    }

    MentatTextFile::MentatEntry& mentatEntry = mentatEntries[index];

    if(mentatEntry.menuLevel != 1) {
        return;
    }

    int animID;
    std::string text;
    std::string name;

	int missionnumber = ((mission+1)/3)+1;

    if(mentatEntry.filename == "0") {
        animID = getMissionSpecificAnim(missionnumber);
		text = pTextManager->getBriefingText(missionnumber, MISSION_DESCRIPTION, house);
    } else if(mentatEntry.filename == "3") {
        animID = getMissionSpecificAnim(missionnumber);
		text = pTextManager->getBriefingText(missionnumber, MISSION_ADVICE, house);
    } else {
        animID = getAnimByFilename(mentatEntry.filename);
        text = mentatEntry.content;
        name = mentatEntry.name;
    }

    Anim.SetAnimation(pGFXManager->getAnimation(animID));
    setText(text);
    ItemDescriptionLabel.SetText(name);

    Anim.SetVisible(true);
    Anim.SetEnabled(true);
    ItemDescriptionLabel.SetVisible(true);

    MentatTopicsList.SetVisible(false);
    MentatTopicsList.SetEnabled(false);
    ExitButton.SetVisible(false);
    ExitButton.SetEnabled(false);
}
