
#include <Menu/MultiPlayerMenu.h>
#include <Menu/CustomGameMenu.h>
#include <Menu/CustomGamePlayers.h>

#include <FileClasses/GFXManager.h>
#include <FileClasses/TextManager.h>

#include <Network/NetworkManager.h>
#include <Network/ENetHelper.h>

#include <GUI/MsgBox.h>

#include <globals.h>

#include <misc/string_util.h>

MultiPlayerMenu::MultiPlayerMenu() : MenuBase() {
	// set up window
	SDL_Surface *surf;
	surf = pGFXManager->getUIGraphic(UI_MenuBackground);

	SetBackground(surf,false);
	Resize(surf->w,surf->h);

	SetWindowWidget(&WindowWidget);

	WindowWidget.AddWidget(&MainVBox, Point(24,23),	Point(screen->w - 48, screen->h - 46));

    CaptionLabel.SetText("Multiplayer Game");
    CaptionLabel.SetAlignment(Alignment_HCenter);
    MainVBox.AddWidget(&CaptionLabel, 24);
    MainVBox.AddWidget(VSpacer::Create(24));

	ConnectHBox.AddWidget(Label::Create("Host:"), 50);
	TextBox_ConnectHost.SetText("localhost");
	ConnectHBox.AddWidget(&TextBox_ConnectHost);
	ConnectHBox.AddWidget(HSpacer::Create(20));
	ConnectHBox.AddWidget(Label::Create("Port:"), 50);
	TextBox_ConnectPort.SetText(stringify(DEFAULT_PORT));
	ConnectHBox.AddWidget(&TextBox_ConnectPort, 90);
	ConnectHBox.AddWidget(HSpacer::Create(20));
	Button_Connect.SetText(pTextManager->getLocalized("Connect"));
	Button_Connect.SetOnClick(std::bind(&MultiPlayerMenu::OnConnect, this));
	ConnectHBox.AddWidget(&Button_Connect, 100);

    MainVBox.AddWidget(Spacer::Create(), 0.05);
	MainVBox.AddWidget(&ConnectHBox, 28);

	MainVBox.AddWidget(VSpacer::Create(16));

    MainVBox.AddWidget(Spacer::Create(), 0.05);
    MainVBox.AddWidget(&MainHBox, 0.85);

    MainHBox.AddWidget(&LeftVBox, 180);

	Button_CreateLANGame.SetText(pTextManager->getLocalized("Create LAN Game"));
	Button_CreateLANGame.SetOnClick(std::bind(&MultiPlayerMenu::OnCreateLANGame, this));
	LeftVBox.AddWidget(&Button_CreateLANGame, 0.1);

	LeftVBox.AddWidget(VSpacer::Create(8));

	Button_CreateInternetGame.SetText(pTextManager->getLocalized("Create Internet Game"));
	Button_CreateInternetGame.SetOnClick(std::bind(&MultiPlayerMenu::OnCreateInternetGame, this));
	LeftVBox.AddWidget(&Button_CreateInternetGame, 0.1);

	LeftVBox.AddWidget(Spacer::Create(), 0.8);

    RightVBox.AddWidget(&GameTypeButtonsHBox, 24);

    MainHBox.AddWidget(HSpacer::Create(8));
    MainHBox.AddWidget(Spacer::Create(), 0.05);

    LANGamesButton.SetText(pTextManager->getLocalized("LAN Games"));
    LANGamesButton.SetToggleButton(true);
    LANGamesButton.SetOnClick(std::bind(&MultiPlayerMenu::OnGameTypeChange, this, 0));
    GameTypeButtonsHBox.AddWidget(&LANGamesButton, 0.35);

    InternetGamesButton.SetText(pTextManager->getLocalized("Internet Games"));
    InternetGamesButton.SetToggleButton(true);
    InternetGamesButton.SetOnClick(std::bind(&MultiPlayerMenu::OnGameTypeChange, this, 1));
    GameTypeButtonsHBox.AddWidget(&InternetGamesButton, 0.35);

	GameTypeButtonsHBox.AddWidget(Spacer::Create(), 0.3);

    GameList.setAutohideScrollbar(false);
    GameList.SetOnSelectionChange(std::bind(&MultiPlayerMenu::OnGameListSelectionChange, this, std::placeholders::_1));
    GameList.SetOnDoubleClick(std::bind(&MultiPlayerMenu::OnJoin, this));
    RightVBox.AddWidget(&GameList, 0.95);

    MainHBox.AddWidget(&RightVBox, 0.9);
    MainHBox.AddWidget(Spacer::Create(), 0.05);

    MainVBox.AddWidget(Spacer::Create(), 0.05);
    MainVBox.AddWidget(VSpacer::Create(10));

	MainVBox.AddWidget(&ButtonHBox, 24);

    ButtonHBox.AddWidget(HSpacer::Create(70));
    Button_Back.SetText(pTextManager->getLocalized("Back"));
	Button_Back.SetOnClick(std::bind(&MultiPlayerMenu::OnQuit, this));
	ButtonHBox.AddWidget(&Button_Back, 0.1);

	ButtonHBox.AddWidget(Spacer::Create(), 0.8);

    Button_Join.SetText(pTextManager->getLocalized("Join"));
	Button_Join.SetOnClick(std::bind(&MultiPlayerMenu::OnJoin, this));
	ButtonHBox.AddWidget(&Button_Join, 0.1);
	ButtonHBox.AddWidget(HSpacer::Create(90));

	// Start Network Manager
	fprintf(stdout, "starting network..."); fflush(stdout);
    pNetworkManager = new NetworkManager(settings.Network.ServerPort, settings.Network.MetaServer);
	LANGameFinderAndAnnouncer* pLANGFAA = pNetworkManager->getLANGameFinderAndAnnouncer();
	pLANGFAA->SetOnNewServer(std::bind(&MultiPlayerMenu::OnNewLANServer, this, std::placeholders::_1));
	pLANGFAA->SetOnUpdateServer(std::bind(&MultiPlayerMenu::OnUpdateLANServer, this, std::placeholders::_1));
	pLANGFAA->SetOnRemoveServer(std::bind(&MultiPlayerMenu::OnRemoveLANServer, this, std::placeholders::_1));
	pLANGFAA->refreshServerList();
    fprintf(stdout, "\t\tfinished\n"); fflush(stdout);

	OnGameTypeChange(0);
}


MultiPlayerMenu::~MultiPlayerMenu() {
	fprintf(stdout, "stopping network..."); fflush(stdout);
	delete pNetworkManager;
	pNetworkManager = NULL;
    fprintf(stdout, "\t\tfinished\n");
}


/**
	This method is called, when the child window is about to be closed.
	This child window will be closed after this method returns.
	\param	pChildWindow	The child window that will be closed
*/
void MultiPlayerMenu::OnChildWindowClose(Window* pChildWindow) {
    // Connection canceled
    // TODO
}

void MultiPlayerMenu::OnCreateLANGame() {
    CustomGameMenu* pCustomGameMenu = new CustomGameMenu(true, true);
    pCustomGameMenu->showMenu();
    delete pCustomGameMenu;
}


void MultiPlayerMenu::OnCreateInternetGame() {
    CustomGameMenu* pCustomGameMenu = new CustomGameMenu(true, false);
    pCustomGameMenu->showMenu();
    delete pCustomGameMenu;
}


void MultiPlayerMenu::OnConnect() {
	std::string hostname = TextBox_ConnectHost.GetText();
	int port = atol(TextBox_ConnectPort.GetText().c_str());

    pNetworkManager->SetOnReceiveGameInfo(std::bind(&MultiPlayerMenu::OnReceiveGameInfo, this, std::placeholders::_1, std::placeholders::_2));
    pNetworkManager->SetOnPeerDisconnected(std::bind(&MultiPlayerMenu::OnPeerDisconnected, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3));
    pNetworkManager->connect(hostname, port, settings.General.PlayerName);

    OpenWindow(MsgBox::Create(pTextManager->getLocalized("Connecting...")));
}


void MultiPlayerMenu::OnPeerDisconnected(std::string playername, bool bHost, int cause) {
    if(bHost) {
        pNetworkManager->SetOnReceiveGameInfo(std::function<void (GameInitSettings, ChangeEventList)>());
        pNetworkManager->SetOnPeerDisconnected(std::function<void (std::string, bool, int)>());
        CloseChildWindow();

        showDisconnectMessageBox(cause);
    }
}

void MultiPlayerMenu::OnJoin() {
	int selectedEntry = GameList.GetSelectedIndex();
	if(selectedEntry >= 0) {
		GameServerInfo* pGameServerInfo = (GameServerInfo*) GameList.GetEntryPtrData(selectedEntry);

        pNetworkManager->SetOnReceiveGameInfo(std::bind(&MultiPlayerMenu::OnReceiveGameInfo, this, std::placeholders::_1, std::placeholders::_2));
        pNetworkManager->SetOnPeerDisconnected(std::bind(&MultiPlayerMenu::OnPeerDisconnected, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3));
        pNetworkManager->connect(pGameServerInfo->serverAddress, settings.General.PlayerName);

        OpenWindow(MsgBox::Create(pTextManager->getLocalized("Connecting...")));
	}
}


void MultiPlayerMenu::OnQuit() {
	SDL_Event quitEvent;
	quitEvent.type = SDL_QUIT;
	SDL_PushEvent(&quitEvent);
}


void MultiPlayerMenu::OnGameTypeChange(int buttonID) {
    MetaServerClient* pMetaServerClient = pNetworkManager->getMetaServerClient();
    if((buttonID == 0) && InternetGamesButton.GetToggleState() == true) {
        // LAN Games

        GameList.ClearAllEntries();
        InternetGameList.clear();

        std::list<GameServerInfo>::iterator iter;
        for(iter = LANGameList.begin(); iter != LANGameList.end(); ++iter) {
            GameServerInfo& gameServerInfo = *iter;
        	std::string description = gameServerInfo.serverName + " (" + Address2String(gameServerInfo.serverAddress) + " : " + stringify(gameServerInfo.serverAddress.port) + ") - "
                                + gameServerInfo.mapName + " (" + stringify(gameServerInfo.numPlayers) + "/" + stringify(gameServerInfo.maxPlayers) + ")";
            GameList.AddEntry(description, &*iter);
        }

        // stop listening on internet games
        pMetaServerClient->SetOnGameServerInfoList(std::function<void (std::list<GameServerInfo>&)>());
        pMetaServerClient->SetOnMetaServerError(std::function<void (int, std::string)>());
    } else if((buttonID == 1) && (LANGamesButton.GetToggleState() == true)) {
        // Internet Games

        GameList.ClearAllEntries();
        InternetGameList.clear();

        // start listening on internet games
        pMetaServerClient->SetOnGameServerInfoList(std::bind(&MultiPlayerMenu::OnGameServerInfoList, this, std::placeholders::_1));
        pMetaServerClient->SetOnMetaServerError(std::bind(&MultiPlayerMenu::OnMetaServerError, this, std::placeholders::_1, std::placeholders::_2));
    }

    LANGamesButton.SetToggleState(buttonID == 0);
    InternetGamesButton.SetToggleState(buttonID == 1);

}


void MultiPlayerMenu::OnGameListSelectionChange(bool bInteractive) {
}


void MultiPlayerMenu::OnNewLANServer(GameServerInfo gameServerInfo) {
	LANGameList.push_back(gameServerInfo);
	std::string description = gameServerInfo.serverName + " (" + Address2String(gameServerInfo.serverAddress) + " : " + stringify(gameServerInfo.serverAddress.port) + ") - "
                                + gameServerInfo.mapName + " (" + stringify(gameServerInfo.numPlayers) + "/" + stringify(gameServerInfo.maxPlayers) + ")";
	GameList.AddEntry(description, &LANGameList.back());
}

void MultiPlayerMenu::OnUpdateLANServer(GameServerInfo gameServerInfo) {
    size_t index = 0;
    std::list<GameServerInfo>::iterator iter;
    for(iter = LANGameList.begin(); iter != LANGameList.end(); ++iter) {
        if(*iter == gameServerInfo) {
            *iter = gameServerInfo;
            break;
        }

        index++;
    }

	if(index < LANGameList.size()) {
        std::string description = gameServerInfo.serverName + " (" + Address2String(gameServerInfo.serverAddress) + " : " + stringify(gameServerInfo.serverAddress.port) + ") - "
                                    + gameServerInfo.mapName + " (" + stringify(gameServerInfo.numPlayers) + "/" + stringify(gameServerInfo.maxPlayers) + ")";

        GameList.SetEntry(index, description);
	}
}

void MultiPlayerMenu::OnRemoveLANServer(GameServerInfo gameServerInfo) {
	for(int i=0;i<GameList.GetNumEntries();i++) {
		GameServerInfo* pGameServerInfo = (GameServerInfo*) GameList.GetEntryPtrData(i);
		if(*pGameServerInfo == gameServerInfo) {
			GameList.RemoveEntry(i);
			break;
		}
	}

	LANGameList.remove(gameServerInfo);
}

void MultiPlayerMenu::OnGameServerInfoList(std::list<GameServerInfo>& gameServerInfoList) {
    // remove all game servers from the list that are not included in the sent list
    std::list<GameServerInfo>::iterator oldListIter = InternetGameList.begin();
    int index = 0;
    while(oldListIter != InternetGameList.end()) {
        GameServerInfo& gameServerInfo = *oldListIter;

        if(std::find(gameServerInfoList.begin(), gameServerInfoList.end(), gameServerInfo) == gameServerInfoList.end()) {
            // not found => remove
            GameList.RemoveEntry(index);
            oldListIter = InternetGameList.erase(oldListIter);
        } else {
            // found => move on
            ++oldListIter;
            ++index;
        }
    }

    // now add all servers that are included for the first time and update the others
    std::list<GameServerInfo>::iterator newListIter;
    for(newListIter = gameServerInfoList.begin(); newListIter != gameServerInfoList.end(); ++newListIter) {
        GameServerInfo& gameServerInfo = *newListIter;

        size_t oldListIndex = 0;
        std::list<GameServerInfo>::iterator oldListIter;
        for(oldListIter = InternetGameList.begin(); oldListIter != InternetGameList.end(); ++oldListIter) {
            if(*oldListIter == gameServerInfo) {
                // found => update
                *oldListIter = gameServerInfo;
                break;
            }

            oldListIndex++;
        }


        if(oldListIndex >= InternetGameList.size()) {
            // not found => add at the end
            InternetGameList.push_back(gameServerInfo);
            std::string description = gameServerInfo.serverName + " (" + Address2String(gameServerInfo.serverAddress) + " : " + stringify(gameServerInfo.serverAddress.port) + ") - "
                                + gameServerInfo.mapName + " (" + stringify(gameServerInfo.numPlayers) + "/" + stringify(gameServerInfo.maxPlayers) + ")";
            GameList.AddEntry(description, &InternetGameList.back());
        } else {
            // found => update
            std::string description = gameServerInfo.serverName + " (" + Address2String(gameServerInfo.serverAddress) + " : " + stringify(gameServerInfo.serverAddress.port) + ") - "
                                    + gameServerInfo.mapName + " (" + stringify(gameServerInfo.numPlayers) + "/" + stringify(gameServerInfo.maxPlayers) + ")";

            GameList.SetEntry(oldListIndex, description);
        }

    }
}

void MultiPlayerMenu::OnMetaServerError(int errorcause, std::string errorMessage) {
    switch(errorcause) {
        case METASERVERCOMMAND_ADD: {
            OpenWindow(MsgBox::Create("MetaServer error on adding game server:\n" + errorMessage));
        } break;

        case METASERVERCOMMAND_UPDATE: {
            OpenWindow(MsgBox::Create("MetaServer error on updating game server:\n" + errorMessage));
        } break;

        case METASERVERCOMMAND_REMOVE: {
            OpenWindow(MsgBox::Create("MetaServer error on removing game server:\n" + errorMessage));
        } break;

        case METASERVERCOMMAND_LIST	: {
            OpenWindow(MsgBox::Create("MetaServer error on list game servers:\n" + errorMessage));
        } break;

        case METASERVERCOMMAND_EXIT:
        default: {
            OpenWindow(MsgBox::Create("MetaServer error:\n" + errorMessage));
        } break;
    }
}

void MultiPlayerMenu::OnReceiveGameInfo(GameInitSettings gameInitSettings, ChangeEventList changeEventList) {
    CloseChildWindow();

    pNetworkManager->SetOnPeerDisconnected(std::function<void (std::string, bool, int)>());

    CustomGamePlayers* pCustomGamePlayers = new CustomGamePlayers(gameInitSettings, false);
    pCustomGamePlayers->OnReceiveChangeEventList(changeEventList);
    int ret = pCustomGamePlayers->showMenu();
    delete pCustomGamePlayers;
    switch(ret) {
        case MENU_QUIT_DEFAULT: {
            // nothing
        } break;

        case MENU_QUIT_GAME_FINISHED: {
            quit(MENU_QUIT_GAME_FINISHED);
        } break;

        default: {
            showDisconnectMessageBox(ret);
        } break;
    }
}

void MultiPlayerMenu::showDisconnectMessageBox(int cause) {
    switch(cause) {
        case NETWORKDISCONNECT_QUIT: {
            OpenWindow(MsgBox::Create(pTextManager->getLocalized("The game host quit the game!")));
        } break;

        case NETWORKDISCONNECT_TIMEOUT: {
            OpenWindow(MsgBox::Create(pTextManager->getLocalized("The server timed out!")));
        } break;

        case NETWORKDISCONNECT_PLAYER_EXISTS: {
            OpenWindow(MsgBox::Create(pTextManager->getLocalized("A player with the same name as yours already exists!")));
        } break;

        case NETWORKDISCONNECT_GAME_FULL: {
            OpenWindow(MsgBox::Create(pTextManager->getLocalized("There is no free player slot in this game left!")));
        } break;

        default: {
            OpenWindow(MsgBox::Create(pTextManager->getLocalized("The connection to the game host was lost!")));
        } break;
    }
}
