/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <Menu/OptionsMenu.h>

#include <globals.h>

#include <GUI/Label.h>
#include <GUI/Spacer.h>
#include <GUI/dune/GameOptionsWindow.h>
#include <GUI/MsgBox.h>

#include <main.h>

#include <FileClasses/GFXManager.h>
#include <FileClasses/TextManager.h>
#include <FileClasses/INIFile.h>
#include <FileClasses/music/MusicPlayer.h>

#include <SoundPlayer.h>

#include <string.h>
#include <algorithm>

OptionsMenu::OptionsMenu() : MenuBase()
{
    determineAvailableScreenResolutions();

    currentGameOptions = settings.GameOptions;

	// set up window
	SDL_Surface *surf;
	surf = pGFXManager->getUIGraphic(UI_MenuBackground);

	SetBackground(surf,false);
	Resize(surf->w,surf->h);

	SetWindowWidget(&WindowWidget);

	WindowWidget.AddWidget(&MainVBox, Point(70,50), Point(GetSize().x - 140,GetSize().y - 100));

    MainVBox.AddWidget(Spacer::Create(), 0.2);

	NameHBox.AddWidget(Spacer::Create(), 0.5);
	NameHBox.AddWidget(Label::Create(pTextManager->getLocalized("Player Name")), 190);
	TextBox_Name.SetMaximumTextLength(30);
	TextBox_Name.SetOnTextChange(std::bind(&OptionsMenu::OnChangeOption, this, true));
	NameHBox.AddWidget(&TextBox_Name, 250);
	NameHBox.AddWidget(Spacer::Create(), 0.5);
	TextBox_Name.SetText(settings.General.PlayerName);

	MainVBox.AddWidget(&NameHBox, 0.01);

	MainVBox.AddWidget(VSpacer::Create(5));

    GameOptionsHBox.AddWidget(Spacer::Create(), 0.5);

    GameOptionsHBox.AddWidget(Label::Create(pTextManager->getLocalized("Default Game Options")), 190);
    GameOptionsButton.SetText(pTextManager->getLocalized("Change..."));
    GameOptionsButton.SetOnClick(std::bind(&OptionsMenu::OnGameOptions, this));
    GameOptionsHBox.AddWidget(&GameOptionsButton, 130);

    GameOptionsHBox.AddWidget(Spacer::Create(), 120);

    GameOptionsHBox.AddWidget(Spacer::Create(), 0.5);

	MainVBox.AddWidget(&GameOptionsHBox, 0.01);

	MainVBox.AddWidget(Spacer::Create(), 0.2);

	LanguageHBox.AddWidget(Spacer::Create(), 0.5);
	LanguageHBox.AddWidget(Label::Create(pTextManager->getLocalized("Language")), 190);
	DropDownBox_Language.AddEntry("English", LNG_ENG);
	DropDownBox_Language.AddEntry("French", LNG_FRE);
	DropDownBox_Language.AddEntry("German", LNG_GER);
	switch(settings.General.Language) {
        case LNG_ENG: DropDownBox_Language.SetSelectedItem(0); break;
        case LNG_FRE: DropDownBox_Language.SetSelectedItem(1); break;
        case LNG_GER: DropDownBox_Language.SetSelectedItem(2); break;
	}
	DropDownBox_Language.SetOnSelectionChange(std::bind(&OptionsMenu::OnChangeOption, this, std::placeholders::_1));
	LanguageHBox.AddWidget(&DropDownBox_Language, 100);
	LanguageHBox.AddWidget(Spacer::Create(), 150);
	LanguageHBox.AddWidget(Spacer::Create(), 0.5);

	MainVBox.AddWidget(&LanguageHBox, 0.01);

	MainVBox.AddWidget(VSpacer::Create(1));

	GeneralHBox.AddWidget(Spacer::Create(), 0.5);
	Checkbox_Intro.SetText(pTextManager->getLocalized("Play Intro"));
	Checkbox_Intro.SetChecked(settings.General.PlayIntro);
	Checkbox_Intro.SetOnClick(std::bind(&OptionsMenu::OnChangeOption, this, true));
    GeneralHBox.AddWidget(&Checkbox_Intro, 440);
    GeneralHBox.AddWidget(Spacer::Create(), 0.5);

    MainVBox.AddWidget(&GeneralHBox, 0.01);

	MainVBox.AddWidget(Spacer::Create(), 0.2);

	ResolutionHBox.AddWidget(Spacer::Create(), 0.5);
	ResolutionHBox.AddWidget(Label::Create(pTextManager->getLocalized("Video Resolution")), 190);

    std::vector<Coord>::const_iterator iter;
    int i = 0;
    for(iter = availScreenRes.begin(); iter != availScreenRes.end(); ++iter, ++i) {
        char temp[20];
        sprintf(temp,"%d x %d", iter->x, iter->y);
        DropDownBox_Resolution.AddEntry(temp, i);
        if(iter->x == settings.Video.Width && iter->y == settings.Video.Height) {
            DropDownBox_Resolution.SetSelectedItem(i);
        }
    }
    DropDownBox_Resolution.SetOnSelectionChange(std::bind(&OptionsMenu::OnChangeOption, this, std::placeholders::_1));
	ResolutionHBox.AddWidget(&DropDownBox_Resolution, 100);
	ResolutionHBox.AddWidget(Spacer::Create(), 15);
	DropDownBox_Zoomlevel.AddEntry("Zoomlevel x1", 0);
	DropDownBox_Zoomlevel.AddEntry("Zoomlevel x2", 1);
	DropDownBox_Zoomlevel.AddEntry("Zoomlevel x3", 2);
	DropDownBox_Zoomlevel.SetSelectedItem(settings.Video.PreferredZoomLevel);
	DropDownBox_Zoomlevel.SetOnSelectionChange(std::bind(&OptionsMenu::OnChangeOption, this, std::placeholders::_1));
	ResolutionHBox.AddWidget(&DropDownBox_Zoomlevel, 105);
	ResolutionHBox.AddWidget(Spacer::Create(), 30);
	ResolutionHBox.AddWidget(Spacer::Create(), 0.5);

	MainVBox.AddWidget(&ResolutionHBox, 0.01);

    MainVBox.AddWidget(VSpacer::Create(1));

    VideoHBox.AddWidget(Spacer::Create(), 0.5);
    Checkbox_FullScreen.SetText(pTextManager->getLocalized("Full Screen"));
    Checkbox_FullScreen.SetChecked(settings.Video.Fullscreen);
    Checkbox_FullScreen.SetOnClick(std::bind(&OptionsMenu::OnChangeOption, this, true));
    VideoHBox.AddWidget(&Checkbox_FullScreen, 220);
    Checkbox_DoubleBuffering.SetText(pTextManager->getLocalized("Double Buffering"));
    Checkbox_DoubleBuffering.SetChecked(settings.Video.DoubleBuffering);
    Checkbox_DoubleBuffering.SetOnClick(std::bind(&OptionsMenu::OnChangeOption, this, true));
    VideoHBox.AddWidget(&Checkbox_DoubleBuffering, 220);
    VideoHBox.AddWidget(Spacer::Create(), 0.5);

    MainVBox.AddWidget(&VideoHBox, 0.01);

	MainVBox.AddWidget(Spacer::Create(), 0.2);

    AudioHBox.AddWidget(Spacer::Create(), 0.5);
    Checkbox_PlaySFX.SetText(pTextManager->getLocalized("Play SFX"));
    Checkbox_PlaySFX.SetChecked(settings.Audio.PlaySFX);
    Checkbox_PlaySFX.SetOnClick(std::bind(&OptionsMenu::OnChangeOption, this, true));
    AudioHBox.AddWidget(&Checkbox_PlaySFX, 220);
    Checkbox_PlayMusic.SetText(pTextManager->getLocalized("Play Music"));
    Checkbox_PlayMusic.SetChecked(settings.Audio.PlayMusic);
    Checkbox_PlayMusic.SetOnClick(std::bind(&OptionsMenu::OnChangeOption, this, true));
    AudioHBox.AddWidget(&Checkbox_PlayMusic, 220);
    AudioHBox.AddWidget(Spacer::Create(), 0.5);

    MainVBox.AddWidget(&AudioHBox, 0.01);

	MainVBox.AddWidget(Spacer::Create(), 0.2);

	NetworkPortHBox.AddWidget(Spacer::Create(), 0.5);
    NetworkPortHBox.AddWidget(Label::Create(pTextManager->getLocalized("Port")), 190);
	TextBox_Port.SetMaximumTextLength(30);
	TextBox_Port.SetOnTextChange(std::bind(&OptionsMenu::OnChangeOption, this, true));
	NetworkPortHBox.AddWidget(&TextBox_Port, 100);
	TextBox_Port.SetText(stringify<int>(settings.Network.ServerPort));
	NetworkPortHBox.AddWidget(Spacer::Create(), 150);
	NetworkPortHBox.AddWidget(Spacer::Create(), 0.5);
	MainVBox.AddWidget(&NetworkPortHBox, 0.01);

	MainVBox.AddWidget(VSpacer::Create(5));

    NetworkMetaServerHBox.AddWidget(Spacer::Create(), 0.5);
    NetworkMetaServerHBox.AddWidget(Label::Create(pTextManager->getLocalized("MetaServer")), 190);
	TextBox_MetaServer.SetOnTextChange(std::bind(&OptionsMenu::OnChangeOption, this, true));
	NetworkMetaServerHBox.AddWidget(&TextBox_MetaServer, 250);
	TextBox_MetaServer.SetText(settings.Network.MetaServer);
	NetworkMetaServerHBox.AddWidget(Spacer::Create(), 0.5);
	MainVBox.AddWidget(&NetworkMetaServerHBox, 0.01);

	MainVBox.AddWidget(Spacer::Create(), 0.2);

	OkCancel_HBox.AddWidget(Spacer::Create());

	Button_Back.SetText(pTextManager->getLocalized("Back"));
	Button_Back.SetOnClick(std::bind(&OptionsMenu::OnOptionsCancel, this));
	OkCancel_HBox.AddWidget(&Button_Back);

	OkCancel_HBox.AddWidget(Spacer::Create());

	Button_Accept.SetText(pTextManager->getLocalized("Accept"));
	Button_Accept.SetVisible(false);
	Button_Accept.SetOnClick(std::bind(&OptionsMenu::OnOptionsOK, this));
	OkCancel_HBox.AddWidget(&Button_Accept);

	OkCancel_HBox.AddWidget(Spacer::Create());

	MainVBox.AddWidget(&OkCancel_HBox, 26);

    MainVBox.AddWidget(Spacer::Create(), 0.1);
}

OptionsMenu::~OptionsMenu()
{
	;
}

void OptionsMenu::OnChangeOption(bool bInteractive) {
    bool bChanged = false;

    bChanged |= (settings.General.PlayerName != TextBox_Name.GetText());
    bChanged |= (settings.General.Language != DropDownBox_Language.GetSelectedEntryIntData());
    bChanged |= (settings.General.PlayIntro != Checkbox_Intro.IsChecked());

    int selectedResolution = DropDownBox_Resolution.GetSelectedEntryIntData();
    bChanged |= (settings.Video.Width != availScreenRes[selectedResolution].x);
    bChanged |= (settings.Video.Height != availScreenRes[selectedResolution].y);
    bChanged |= (settings.Video.PreferredZoomLevel != DropDownBox_Zoomlevel.GetSelectedEntryIntData());
    bChanged |= (settings.Video.Fullscreen != Checkbox_FullScreen.IsChecked());
    bChanged |= (settings.Video.DoubleBuffering != Checkbox_DoubleBuffering.IsChecked());

    bChanged |= (settings.Audio.PlaySFX != Checkbox_PlaySFX.IsChecked());
    bChanged |= (settings.Audio.PlayMusic != Checkbox_PlayMusic.IsChecked());

    bChanged |= (settings.GameOptions.ConcreteRequired != currentGameOptions.ConcreteRequired);
    bChanged |= (settings.GameOptions.StructuresDegradeOnConcrete != currentGameOptions.StructuresDegradeOnConcrete);
    bChanged |= (settings.GameOptions.FogOfWar != currentGameOptions.FogOfWar);
    bChanged |= (settings.GameOptions.StartWithExploredMap != currentGameOptions.StartWithExploredMap);
    bChanged |= (settings.GameOptions.InstantBuild != currentGameOptions.InstantBuild);

    bChanged |= (settings.Network.ServerPort != atoi(TextBox_Port.GetText().c_str()));
    bChanged |= (settings.Network.MetaServer != TextBox_MetaServer.GetText());

    Button_Accept.SetVisible(bChanged);
}

void OptionsMenu::OnOptionsOK() {
    std::string playername = TextBox_Name.GetText();
    if(playername.empty()) {
        OpenWindow(MsgBox::Create(pTextManager->getLocalized("Please enter a Player Name.")));
        return;
    }

    int serverport;
    if(!parseString(TextBox_Port.GetText(), serverport) || serverport <= 0 || serverport > 65535) {
        OpenWindow(MsgBox::Create(strprintf(pTextManager->getLocalized("Server Port must be between 1 and 65535!\nDefault Server Port is %d!").c_str(), DEFAULT_PORT)));
        return;
    }

    std::string metaserver = TextBox_MetaServer.GetText();
    if(metaserver.empty()) {
        OpenWindow(MsgBox::Create(pTextManager->getLocalized("Please enter a MetaServer.")));
        return;
    }

	settings.General.PlayerName = playername;

    settings.General.setLanguage(DropDownBox_Language.GetSelectedEntryIntData());
	settings.General.PlayIntro = Checkbox_Intro.IsChecked();

    int selectedResolution = DropDownBox_Resolution.GetSelectedEntryIntData();
    settings.Video.Width = availScreenRes[selectedResolution].x;
	settings.Video.Height = availScreenRes[selectedResolution].y;
	int selectedZoomlevel = DropDownBox_Zoomlevel.GetSelectedEntryIntData();
	settings.Video.PreferredZoomLevel = selectedZoomlevel;
	settings.Video.Fullscreen = Checkbox_FullScreen.IsChecked();
	settings.Video.DoubleBuffering = Checkbox_DoubleBuffering.IsChecked();

	settings.Audio.PlaySFX = Checkbox_PlaySFX.IsChecked();
	settings.Audio.PlayMusic = Checkbox_PlayMusic.IsChecked();

	settings.GameOptions = currentGameOptions;

	settings.Network.ServerPort = serverport;
	settings.Network.MetaServer = metaserver;

	SaveConfiguration2File();

	// sound is not reinitialized when restarting
	// => music and sound player do not reload settings
	soundPlayer->setSound(settings.Audio.PlaySFX);
	if(musicPlayer->isMusicOn() != settings.Audio.PlayMusic) {
        musicPlayer->setMusic(settings.Audio.PlayMusic);
        musicPlayer->changeMusic(MUSIC_INTRO);
	}

	quit(MENU_QUIT_REINITIALIZE);
}

void OptionsMenu::OnOptionsCancel() {
	quit();
}

void OptionsMenu::OnGameOptions() {
    OpenWindow(GameOptionsWindow::Create(currentGameOptions));
}

void OptionsMenu::SaveConfiguration2File() {
	INIFile myINIFile(GetConfigFilepath());

	myINIFile.setBoolValue("General","Play Intro",settings.General.PlayIntro);
	myINIFile.setIntValue("Video","Width",settings.Video.Width);
	myINIFile.setIntValue("Video","Height",settings.Video.Height);
	myINIFile.setBoolValue("Video","Fullscreen",settings.Video.Fullscreen);
	myINIFile.setBoolValue("Video","Double Buffering",settings.Video.DoubleBuffering);
	myINIFile.setIntValue("Video","Preferred Zoom Level",settings.Video.PreferredZoomLevel);
	myINIFile.setStringValue("General","Player Name",settings.General.PlayerName);
	myINIFile.setStringValue("General","Language",settings.General.getLanguageString());
	myINIFile.setBoolValue("Audio","Play SFX",settings.Audio.PlaySFX);
	myINIFile.setBoolValue("Audio","Play Music",settings.Audio.PlayMusic);
	myINIFile.setBoolValue("Game Options","Concrete Required",settings.GameOptions.ConcreteRequired);
    myINIFile.setBoolValue("Game Options","Structures Degrade On Concrete",settings.GameOptions.StructuresDegradeOnConcrete);
    myINIFile.setBoolValue("Game Options","Fog of War",settings.GameOptions.FogOfWar);
    myINIFile.setBoolValue("Game Options","Start with Explored Map",settings.GameOptions.StartWithExploredMap);
    myINIFile.setBoolValue("Game Options","Instant Build",settings.GameOptions.InstantBuild);
    myINIFile.setIntValue("Network","ServerPort",settings.Network.ServerPort);
    myINIFile.setStringValue("Network","MetaServer",settings.Network.MetaServer);

	myINIFile.SaveChangesTo(GetConfigFilepath());
}

void OptionsMenu::determineAvailableScreenResolutions() {
    availScreenRes.clear();

	SDL_Rect** pResolutions = SDL_ListModes(NULL , SDL_HWSURFACE | SDL_FULLSCREEN);

	if(pResolutions == NULL || pResolutions == (SDL_Rect**) -1) {
		// Not possible or not available
		// try some standard resolutions

		availScreenRes.push_back( Coord(640, 480) );    // VGA (4:3)
		availScreenRes.push_back( Coord(800, 480) );    // WVGA (5:3)
		availScreenRes.push_back( Coord(800, 600) );    // SVGA (4:3)
		availScreenRes.push_back( Coord(960, 540) );    // Quarter HD (16:9)
		availScreenRes.push_back( Coord(960, 720) );    // ? (4:3)
		availScreenRes.push_back( Coord(1024, 576) );   // WSVGA (16:9)
		availScreenRes.push_back( Coord(1024, 640) );   // ? (16:10)
		availScreenRes.push_back( Coord(1024, 768) );   // XGA (4:3)
		availScreenRes.push_back( Coord(1152, 864) );   // XGA+ (4:3)
		availScreenRes.push_back( Coord(1280, 720 ) );  // WXGA (16:9)
		availScreenRes.push_back( Coord(1280, 768) );   // WXGA (5:3)
		availScreenRes.push_back( Coord(1280, 800) );   // WXGA (16:10)
		availScreenRes.push_back( Coord(1280, 960) );   // SXGA- (4:3)
		availScreenRes.push_back( Coord(1280, 1024) );  // SXGA (5:4)
		availScreenRes.push_back( Coord(1366, 768) );   // HDTV 720p (~16:9)
		availScreenRes.push_back( Coord(1400, 1050) );  // SXGA+ (4:3)
		availScreenRes.push_back( Coord(1440, 900) );   // WXGA+ (16:10)
		availScreenRes.push_back( Coord(1440, 1080) );  // ? (4:3)
		availScreenRes.push_back( Coord(1600, 1200) );  // UXGA (4:3)
		availScreenRes.push_back( Coord(1680, 1050) );  // WSXGA+ (16:10)
		availScreenRes.push_back( Coord(1920, 1080) );  // 1080p (16:9)
		availScreenRes.push_back( Coord(1920, 1200) );  // WUXGA (16:10)
	} else {
	    // step backward through resolutions
		for(int i=0; pResolutions[i] != NULL;  i++) {
		    if(pResolutions[i]->w >= SCREEN_MIN_WIDTH && pResolutions[i]->h >= SCREEN_MIN_HEIGHT) {
		        Coord newRes = Coord(pResolutions[i]->w,pResolutions[i]->h);
		        if(std::find(availScreenRes.begin(), availScreenRes.end(), newRes) == availScreenRes.end()) {
		            // not yet in the list
		            availScreenRes.insert(availScreenRes.begin(), newRes);
		        }
		    }
		}
	}

	Coord currentRes(settings.Video.Width, settings.Video.Height);

    if(std::find(availScreenRes.begin(), availScreenRes.end(), currentRes) == availScreenRes.end()) {
        // not yet in the list
        availScreenRes.insert(availScreenRes.begin(), currentRes);
    }
}

void OptionsMenu::OnChildWindowClose(Window* pChildWindow) {
    GameOptionsWindow* pGameOptionsWindow = dynamic_cast<GameOptionsWindow*>(pChildWindow);
    if(pGameOptionsWindow != NULL) {
        currentGameOptions = pGameOptionsWindow->getGameOptions();

        OnChangeOption(true);
    }
}
