/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <structures/RocketTurret.h>

#include <globals.h>

#include <FileClasses/GFXManager.h>
#include <FileClasses/SFXManager.h>
#include <House.h>
#include <Game.h>

RocketTurret::RocketTurret(House* newOwner) : TurretBase(newOwner)
{
    RocketTurret::init();

    setHealth(getMaxHealth());
}

RocketTurret::RocketTurret(InputStream& stream) : TurretBase(stream)
{
    RocketTurret::init();
}

void RocketTurret::init() {
	itemID = Structure_RocketTurret;
	owner->incrementStructures(itemID);

	attackSound = Sound_Rocket;
	bulletType = Bullet_Rocket;

	weaponReloadTime = 150;

	GraphicID = ObjPic_RocketTurret;
	graphic = pGFXManager->getObjPic(GraphicID,getOwner()->getHouseID());
	numImagesX = 10;
	numImagesY = 1;
	curAnimFrame = FirstAnimFrame = LastAnimFrame = ((10-drawnAngle) % 8) + 2;
}

RocketTurret::~RocketTurret()
{
}

void RocketTurret::doSpecificStuff()
{
	if(getOwner()->hasPower() || (((currentGame->gameType == GAMETYPE_CAMPAIGN) || (currentGame->gameType == GAMETYPE_SKIRMISH)) && getOwner()->isAI())) {
		TurretBase::doSpecificStuff();
	}
}

bool RocketTurret::canAttack(const ObjectBase* object) const
{
	if ((object != NULL)
		&& ((object->getOwner()->getTeam() != owner->getTeam()) || object->getItemID() == Unit_Sandworm)
		&& object->isVisible(getOwner()->getTeam()))
		return true;
	else
		return false;
}
