/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <structures/WOR.h>

#include <globals.h>

#include <FileClasses/GFXManager.h>
#include <House.h>

WOR::WOR(House* newOwner) : BuilderBase(newOwner)
{
    WOR::init();

    setHealth(getMaxHealth());
}

WOR::WOR(InputStream& stream) : BuilderBase(stream)
{
    WOR::init();
}

void WOR::init()
{
    itemID = Structure_WOR;
	owner->incrementStructures(itemID);

	structureSize.x = 2;
	structureSize.y = 2;

	GraphicID = ObjPic_WOR;
	graphic = pGFXManager->getObjPic(GraphicID,getOwner()->getHouseID());
	numImagesX = 4;
	numImagesY = 1;
	FirstAnimFrame = 2;
	LastAnimFrame = 3;
}

WOR::~WOR()
{
}

void WOR::checkSelectionList()
{
	BuilderBase::checkSelectionList();

	std::list<BuildItem>::iterator iter = BuildList.begin();

	switch(owner->getHouseID()) {
		case(HOUSE_ATREIDES):
		case(HOUSE_ORDOS):
		case(HOUSE_HARKONNEN):
		case(HOUSE_MERCENARY): {
			InsertItem(BuildList, iter, Unit_Trooper);
		} break;

		case(HOUSE_FREMEN): {
			InsertItem(BuildList, iter, Unit_FremenTrooper);
		} break;

		case(HOUSE_SARDAUKAR): {
			InsertItem(BuildList, iter, Unit_SardaukarTrooper);
		} break;

		default: {
		} break;
	}
}
