/*
 *  This file is part of Dune Legacy.
 *
 *  Dune Legacy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Dune Legacy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Dune Legacy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <units/MCV.h>

#include <globals.h>

#include <FileClasses/GFXManager.h>
#include <FileClasses/TextManager.h>
#include <House.h>
#include <Explosion.h>
#include <Game.h>
#include <SoundPlayer.h>
#include <Map.h>

#include <players/HumanPlayer.h>

MCV::MCV(House* newOwner) : GroundUnit(newOwner)
{
    MCV::init();

    setHealth(getMaxHealth());
    attackMode = STOP;
}

MCV::MCV(InputStream& stream) : GroundUnit(stream)
{
    MCV::init();
}

void MCV::init() {
    itemID = Unit_MCV;
    owner->incrementUnits(itemID);

	canAttackStuff = false;

	GraphicID = ObjPic_MCV;
	graphic = pGFXManager->getObjPic(GraphicID,getOwner()->getHouseID());

	numImagesX = NUM_ANGLES;
	numImagesY = 1;
}

MCV::~MCV() {

}

void MCV::HandleDeployClick() {
	currentGame->GetCommandManager().addCommand(Command(pLocalPlayer->getPlayerID(), CMD_MCV_DEPLOY,objectID));
}

void MCV::DoDeploy() {
    // check if there is enough room for construction yard
	if (currentGameMap->okayToBuildExclusive(getX(), getY(), 2, 2)) {

        // save needed values
		House* pOwner = getOwner();
		Coord newLocation = getLocation();

		// first place construction yard and then destroy MCV, otherwise a player with only MCV left will lose

		// place construction yard (force placing to place on still existing MCV)
		pOwner->placeStructure(NONE, Structure_ConstructionYard, newLocation.x, newLocation.y, true);

		// destroy MCV but with base class method since we want no explosion
		GroundUnit::destroy();

		return;
	} else {
		currentGame->AddToNewsTicker(pTextManager->getLocalized("You cannot deploy here."));
	}
}

void MCV::destroy() {
    if(currentGameMap->cellExists(location) && isVisible()) {
        Coord realPos(lround(realX), lround(realY));
        currentGame->getExplosionList().push_back(new Explosion(Explosion_SmallUnit, realPos, owner->getHouseID()));

        if(isVisible(getOwner()->getTeam()))
            soundPlayer->playSoundAt(Sound_ExplosionSmall,location);
    }

    GroundUnit::destroy();
}
